import { join } from "path";
import { existsSync, writeFileSync } from "fs";
import StreamZip from "node-stream-zip";
import { last_verse } from "@gracious.tech/bible-references";
import { books_ordered } from "../parts/bible.js";
import { clean_dir, concurrent, request, read_json, list_dirs, list_files } from "../parts/utils.js";
async function download_source(recheck, force, trans_id) {
  const manual_sourced = {};
  const door43_sourced = {};
  const ebible_sourced = {};
  const dbl_sourced = {};
  for (const id of list_dirs(join("sources", "bibles"))) {
    if (trans_id && id !== trans_id) {
      continue;
    }
    const meta = read_json(join("sources", "bibles", id, "meta.json"));
    const src_dir = join("sources", "bibles", id, meta.source.format);
    if (!recheck && existsSync(src_dir) && list_files(src_dir).length) {
      continue;
    }
    if (meta.modified && !force) {
      console.warn(`Skipping ${id} as source has been modified (--force if needed)`);
      continue;
    }
    if (meta.source.service === "manual") {
      manual_sourced[id] = meta;
    } else if (meta.source.service === "door43") {
      door43_sourced[id] = meta;
    } else if (meta.source.service === "ebible") {
      ebible_sourced[id] = meta;
    } else if (meta.source.service === "dbl") {
      dbl_sourced[id] = meta;
    }
  }
  if ([manual_sourced, door43_sourced, ebible_sourced, dbl_sourced].every((source) => !Object.keys(source).length)) {
    console.error("No translations identified");
  }
  await Promise.all([
    generic_update_sources(manual_sourced),
    generic_update_sources(door43_sourced),
    generic_update_sources(ebible_sourced),
    generic_update_sources(dbl_sourced)
  ]);
}
async function generic_update_sources(sources) {
  await concurrent(Object.entries(sources).map(([id, meta]) => async () => {
    console.info(`Downloading source files for ${id}`);
    try {
      await _update_source(id, meta);
    } catch (error) {
      console.error(`FAILED to update source for: ${id}`);
      console.error(`${error}`);
    }
  }));
}
async function _update_source(id, meta) {
  const src_dir = join("sources", "bibles", id);
  const zip_path = join(src_dir, "source.zip");
  const format_dir = join(src_dir, meta.source.format);
  if (meta.source.url) {
    const zip = await request(meta.source.url, "arrayBuffer");
    writeFileSync(zip_path, Buffer.from(zip));
  }
  clean_dir(format_dir);
  const norm_ext = (name) => name.toLowerCase().split(".").at(-1);
  const extractor = new StreamZip.async({ file: zip_path, skipEntryNameValidation: true });
  const zip_entries = Object.values(await extractor.entries());
  const has_official_ext = !!zip_entries.find((e) => norm_ext(e.name) === meta.source.format);
  const src_ext = has_official_ext ? meta.source.format : { usfm: "sfm", usx: "xml" }[meta.source.format];
  for (const entry of zip_entries) {
    if (norm_ext(entry.name) !== src_ext) {
      continue;
    }
    const contents = await extractor.entryData(entry);
    const contents_str = contents.toString("utf-8");
    let book = void 0;
    if (meta.source.format === "usfm") {
      book = /^\\id (\w\w\w)/m.exec(contents_str)?.[1]?.toLowerCase();
    } else if (/<usx/.test(contents_str)) {
      book = /<book[^>]+code="(\w\w\w)"/.exec(contents_str)?.[1]?.toLowerCase();
    }
    if (!book) {
      console.error(`Valid format but couldn't identify book: ${entry.name}`);
      continue;
    } else if (!books_ordered.includes(book)) {
      continue;
    }
    let file_name = `${book}.${meta.source.format}`;
    let regex;
    if (meta.source.format === "usfm") {
      regex = /^\c (\d+)/mg;
    } else {
      regex = /<chapter[^>]+number="(\d+)"/g;
    }
    const chapters = [];
    while (true) {
      const result = regex.exec(contents_str);
      if (!result) {
        break;
      }
      chapters.push([parseInt(result[1] ?? ""), regex.lastIndex]);
    }
    for (let ch = 1; ch <= last_verse[book].length; ch++) {
      if (book === "mal" && ch === 4) {
        continue;
      }
      if (chapters[ch - 1]?.[0] !== ch) {
        file_name += ".invalid";
        break;
      }
      if (meta.source.format !== "usfm") {
        continue;
      }
      const ch_contents = contents_str.slice(chapters[ch - 1][1], chapters[ch]?.[1]);
      const max_line_len = Math.max(...ch_contents.split("\n").map((l) => l.trim().length));
      if (max_line_len < 10) {
        file_name += ".invalid";
        break;
      }
    }
    writeFileSync(join(format_dir, file_name), contents);
  }
}
export {
  download_source
};
//# sourceMappingURL=download.js.map
