import fs from "node:fs";
import { join } from "node:path";
import { execSync } from "node:child_process";
import { usx_to_json_html, usx_to_json_txt } from "usx-to-json";
import { reverse_usx } from "reverse-usx";
import { JSDOM } from "jsdom";
import { pre_usx_to_json } from "../integrations/patches.js";
import { update_manifest } from "../parts/manifest.js";
import { concurrent, PKG_PATH, read_json, write_json, list_files, list_dirs } from "../parts/utils.js";
import { extract_sections, generate_chapter_headings } from "../parts/sections.js";
import { rm_mark_second_ending } from "../parts/mark_ending.js";
const DOM = new JSDOM();
const DOMParser = DOM.window.DOMParser;
const XMLSerializer = DOM.window.XMLSerializer;
async function _source_to_distributable(trans, from, to, src_dir_type = "sources") {
  const src_dir = join(src_dir_type, "bibles", trans, from);
  const dist_dir = join("dist", "bibles", trans, to);
  const valid_src_files = list_files(src_dir).filter((f) => !f.endsWith(".invalid")).length;
  if (valid_src_files === list_files(dist_dir).length) {
    return;
  }
  console.info(`Converting ${trans} to ${to}`);
  let source_is_usx3 = false;
  if (from === "usx") {
    const first_file = list_files(src_dir)[0];
    if (first_file) {
      const contents = fs.readFileSync(join(src_dir, first_file), { encoding: "utf8" });
      const usx_version = /<usx version="([\d.]+)">/.exec(contents);
      source_is_usx3 = parseFloat(usx_version?.[1] ?? "0") >= 3;
    }
  }
  if (from === to && (from === "usfm" || source_is_usx3)) {
    for (const file of list_files(src_dir).filter((f) => !f.endsWith(".invalid"))) {
      fs.copyFileSync(join(src_dir, file), join(dist_dir, file));
    }
    return;
  }
  const from_format = {
    "usx": source_is_usx3 ? "USX3" : "USX",
    "usfm": "USFM"
  }[from];
  const to_format = to === "usfm" ? "USFM" : "USX3";
  const tool = ["usfm", "usx"].includes(from) ? "ParatextConverter" : "";
  const bmc = join(PKG_PATH, "bmc", "BibleMultiConverter-AllInOneEdition.jar");
  const args_fix = "--add-opens java.base/java.lang=ALL-UNNAMED";
  const cmd = `java ${args_fix} "-Dbiblemulticonverter.paratext.usx.verseseparatortext= " -jar ${bmc} ${tool} ${from_format} "${src_dir}" ${to_format} "${dist_dir}" "*.${to}"`;
  try {
    execSync(cmd, { stdio: "ignore" });
  } finally {
    for (const file of list_files(dist_dir)) {
      fs.renameSync(join(dist_dir, file), join(dist_dir, file.toLowerCase()));
    }
  }
}
async function update_dist(trans_id) {
  const force = !!trans_id;
  await concurrent(list_dirs(join("sources", "bibles")).map((id) => async () => {
    if (trans_id && id !== trans_id) {
      return;
    }
    try {
      await _update_dist_single(id, force);
    } catch (error) {
      console.error(`FAILED update dist assets for: ${id}`);
      console.error(error instanceof Error ? error.stack : error);
    }
  }), 4);
  await update_manifest();
}
async function _update_dist_single(id, force) {
  const src_dir = join("sources", "bibles", id);
  const dist_dir = join("dist", "bibles", id);
  const meta_file_path = join(src_dir, "meta.json");
  const meta = read_json(meta_file_path);
  const format_dir = join(src_dir, meta.source.format);
  if (!fs.existsSync(format_dir)) {
    console.warn(`IGNORED ${id} (no source)`);
    return;
  }
  if (force) {
    fs.rmSync(dist_dir, { force: true, recursive: true });
  }
  for (const format of ["usx", "usfm", "html", "txt"]) {
    fs.mkdirSync(join(dist_dir, format), { recursive: true });
  }
  await _source_to_distributable(id, meta.source.format, "usx");
  let usfm_from = meta.source.format;
  let usfm_from_dir = "sources";
  for (const file of list_files(join(dist_dir, "usx"))) {
    const file_path = join(dist_dir, "usx", file);
    const contents = fs.readFileSync(file_path, "utf8");
    let reversed;
    try {
      reversed = reverse_usx(contents, DOMParser, XMLSerializer);
      if (file === "mrk.usx") {
        reversed = rm_mark_second_ending(reversed);
      }
    } catch {
      console.error(`Failed to re-verse ${file_path}`);
    }
    if (contents === reversed) {
      continue;
    } else {
      for (const format of ["usfm", "html", "txt"]) {
        const ext = format === "usfm" ? "usfm" : "json";
        fs.rmSync(join(dist_dir, format, file.replace("usx", ext)), { force: true });
      }
      if (reversed) {
        fs.writeFileSync(file_path, reversed);
      } else {
        fs.unlinkSync(file_path);
      }
      usfm_from = "usx";
      usfm_from_dir = "dist";
    }
  }
  await _source_to_distributable(id, usfm_from, "usfm", usfm_from_dir);
  const usx_dir = join(dist_dir, "usx");
  for (const file of list_files(usx_dir)) {
    const book = file.split(".")[0];
    const src = join(usx_dir, `${book}.usx`);
    const dst_html = join(dist_dir, "html", `${book}.json`);
    const dst_txt = join(dist_dir, "txt", `${book}.json`);
    let usx_str = fs.readFileSync(src, { encoding: "utf8" });
    usx_str = pre_usx_to_json(id, book, usx_str);
    if (!fs.existsSync(dst_txt)) {
      try {
        const txt = usx_to_json_txt(usx_str, DOMParser);
        write_json(dst_txt, txt);
      } catch (error) {
        console.warn(`INVALID BOOK: failed to convert '${book}' to txt for ${id}`);
        console.error(error);
      }
    }
    if (!fs.existsSync(dst_html)) {
      try {
        const html = usx_to_json_html(usx_str, false, DOMParser);
        write_json(dst_html, html);
      } catch (error) {
        console.warn(`INVALID BOOK: failed to convert '${book}' to html for ${id}`);
        console.error(error);
      }
    }
  }
  const trans_extra = { book_names: {}, chapter_headings: {}, sections: {} };
  for (const filename of list_files(join(dist_dir, "txt"))) {
    const book = filename.split(".")[0];
    const file_path = join(dist_dir, "txt", filename);
    const json_txt = JSON.parse(fs.readFileSync(file_path, { encoding: "utf8" }));
    trans_extra.book_names[book] = json_txt.name;
    trans_extra.sections[book] = extract_sections(json_txt);
    trans_extra.chapter_headings[book] = generate_chapter_headings(json_txt, trans_extra.sections[book]);
  }
  write_json(join(dist_dir, "extra.json"), trans_extra);
}
export {
  update_dist
};
//# sourceMappingURL=process.js.map
