import { join } from "path";
import { existsSync } from "fs";
import { books_ordered } from "@gracious.tech/bible-references";
import { read_json, list_dirs, list_files } from "../parts/utils.js";
import { _missing_meta } from "../parts/manifest.js";
function report_items() {
  for (const id of list_dirs(join("sources", "bibles"))) {
    const meta = read_json(join("sources", "bibles", id, "meta.json"));
    let license = meta.copyright.licenses[0]?.license;
    if (!license) {
      license = "none";
    } else if (typeof license === "object") {
      license = "custom";
    }
    let url = "";
    if (meta.source.service === "dbl") {
      url = `https://app.thedigitalbiblelibrary.org/entry?id=${meta.ids.dbl}`;
    } else if (meta.source.service === "ebible") {
      url = `https://ebible.org/Scriptures/details.php?id=${meta.ids.ebible}`;
    }
    const fields = [id, meta.year, license, url];
    console.info(fields.map((field) => `${field ?? "null"}`.padEnd(16)).join(" "));
  }
}
function report_invalid_meta() {
  for (const id of list_dirs(join("sources", "bibles"))) {
    const meta = read_json(join("sources", "bibles", id, "meta.json"));
    if (_missing_meta(meta)) {
      console.error(`MISSING METADATA: ${id}`);
    }
    const service = meta.source.service;
    if (service === "dbl" || service === "ebible") {
      const service_id = meta.ids[service];
      const attr_url = meta.copyright.attribution_url;
      if ((attr_url.includes("ebible") || attr_url.includes("digitalbiblelibrary")) && !meta.copyright.attribution_url.includes(service_id)) {
        console.error(`WRONG ATTRIBUTION: ${id}`);
      }
      if (!meta.source.url.includes(service_id)) {
        console.error(`WRONG ID IN SOURCE URL: ${id}`);
      }
    }
  }
}
function report_invalid_books(report_no_books = false) {
  for (const id of list_dirs(join("sources", "bibles"))) {
    const src_format = existsSync(join("sources", "bibles", id, "usx")) ? "usx" : "usfm";
    const files = list_files(join("sources", "bibles", id, src_format));
    const invalid = files.filter((f) => f.endsWith(".invalid")).map((b) => b.slice(0, 3));
    const num_valid = files.length - invalid.length;
    if (report_no_books) {
      if (num_valid === 0) {
        console.error(`NO BOOKS: ${id}`);
      }
    } else {
      if (invalid.length) {
        console.error(`${id.padEnd(10)}   ${invalid.length.toString().padStart(2)}  ${invalid.slice(0, 5).join(",")}`);
      }
    }
  }
}
function report_incomplete() {
  const ot_books = books_ordered.slice(0, 39);
  const nt_books = books_ordered.slice(39);
  for (const id of list_dirs(join("sources", "bibles"))) {
    const src_format = existsSync(join("sources", "bibles", id, "usx")) ? "usx" : "usfm";
    const files_list = list_files(join("sources", "bibles", id, src_format));
    const valid_books = files_list.filter((f) => !f.endsWith(".invalid")).map((f) => f.slice(0, 3));
    const invalid = files_list.length - valid_books.length;
    let missing_ot = ot_books.filter((b) => !valid_books.includes(b));
    if (missing_ot.length > 5) {
      missing_ot = [];
    }
    let missing_nt = nt_books.filter((b) => !valid_books.includes(b));
    if (missing_nt.length > 5) {
      missing_nt = [];
    }
    if (missing_ot.length || missing_nt.length) {
      console.error(`OT ${missing_ot.length.toString().padStart(2)}   NT ${missing_nt.length.toString().padStart(2)}      INV. ${invalid.toString().padStart(2)}   ${id.padEnd(10)}   ` + [...missing_ot, ...missing_nt].join(","));
    }
  }
}
function report_unprocessed(type) {
  let count = 0;
  for (const id of list_dirs(join("sources", "bibles"))) {
    const src_format = existsSync(join("sources", "bibles", id, "usx")) ? "usx" : "usfm";
    const expected = list_files(join("sources", "bibles", id, src_format)).filter((f) => !f.endsWith(".invalid")).map((b) => b.slice(0, 3));
    const expected_num = expected.length;
    const dist_dir = join("dist", "bibles", id);
    const usfm_num = list_files(join(dist_dir, "usfm")).length;
    const usx_num = list_files(join(dist_dir, "usx")).length;
    const html_num = list_files(join(dist_dir, "html")).length;
    const txt_books = list_files(join(dist_dir, "txt")).map((b) => b.slice(0, 3));
    const txt_num = txt_books.length;
    let min_num = Math.min(usfm_num, usx_num, html_num, txt_num);
    if (type === "usfmx") {
      min_num = Math.min(usfm_num, usx_num);
    } else if (type === "other") {
      if (usx_num < expected_num || usfm_num < expected_num) {
        continue;
      }
      min_num = Math.min(html_num, txt_num);
    }
    if (min_num < expected_num) {
      count++;
      const books = expected.filter((b) => !txt_books.includes(b)).slice(0, 5).join(",");
      console.error(min_num.toString().padStart(2) + " / " + expected_num.toString().padStart(2) + "  " + id.padEnd(10) + books);
    }
  }
  console.info(`
Total: ${count}`);
}
export {
  report_incomplete,
  report_invalid_books,
  report_invalid_meta,
  report_items,
  report_unprocessed
};
//# sourceMappingURL=report.js.map
