import { join } from "node:path";
import { existsSync, mkdirSync } from "node:fs";
import { request, concurrent, write_json, read_json } from "../parts/utils.js";
import { LICENSES } from "../parts/license.js";
import { get_language_data } from "../parts/languages.js";
const IGNORE = [
  "69ffe7eecd894e52"
  // Only second half of genesis
];
const OUTDATED_LANG = { cug: "cnq" };
const org_abbreviations = {
  "545d2cb05f032bce5404dab8": "wft",
  "545d2cb013cb53cafc2bad91": "pbt",
  "545d2cb007eaee5131ab123a": "ubs",
  "545d2cb00fd5dca263562474": "bib",
  "545d2cb00be06579ca809b57": "wbt",
  "54650d065117ad695d428986": "seed",
  "019217aaaebdd5e7e372cf90": "tlf"
};
async function discover(existing, discover_specific_id) {
  const url = "https://app.thedigitalbiblelibrary.org/api/open_access_entries";
  const resp = await request(url, "json");
  let items = resp.entries.filter((item) => item.entrytype === "text");
  items = items.filter((item) => item.rightsHolder[0].name !== "eBible.org");
  const num_total = items.length;
  let num_existing = 0;
  let num_added = 0;
  items = items.filter((item) => !IGNORE.includes(item.id));
  let num_ignored = num_total - items.length;
  const language_data = get_language_data();
  await concurrent(items.map((item) => async () => {
    if (discover_specific_id && item.id !== discover_specific_id) {
      return;
    }
    const org_id = item.rightsHolder[0].link.split("/").at(-1) ?? "";
    const org_abbrev = org_abbreviations[org_id];
    const lang_code = language_data.normalise(
      OUTDATED_LANG[item.languageCode] ?? item.languageCode
    );
    let eng_abbrev = item.nameAbbreviation.toLowerCase().replace(/[^a-z]/gi, "");
    if (lang_code && eng_abbrev.includes(lang_code)) {
      eng_abbrev = eng_abbrev.replace(lang_code, "");
    }
    let trans_id = `${lang_code ?? ""}_${org_abbrev || eng_abbrev || "a"}`;
    if (existing.includes(item.id)) {
      num_existing += 1;
      return;
    }
    if (!lang_code) {
      console.error(`INVALID ${item.languageCode} (unknown language)`);
      num_ignored += 1;
      return;
    }
    let trans_dir = join("sources", "bibles", trans_id);
    let meta_file = join(trans_dir, "meta.json");
    if (existsSync(meta_file)) {
      const existing_meta = read_json(meta_file);
      if (!existing_meta.ids.dbl) {
        existing_meta.ids.dbl = item.id;
        write_json(meta_file, existing_meta, true);
        num_existing += 1;
        return;
      } else {
        trans_id += "_" + eng_abbrev;
        trans_dir = join("sources", "bibles", trans_id);
        meta_file = join(trans_dir, "meta.json");
      }
    }
    const log_ids = `${trans_id}/${item.id}`;
    const html_url = `https://app.thedigitalbiblelibrary.org/entry?id=${item.id}`;
    const html_resp = await request(html_url, "text");
    let license = null;
    let license_url = html_url;
    const cc_abbrev = new Set([...html_resp.matchAll(/cc (by[a-z-]*)/ig)].map((r) => r[1].toLowerCase()));
    if (/public domain/i.test(html_resp) && !/not public domain/i.test(html_resp)) {
      license = "public";
      if (html_resp.includes("creativecommons.org/publicdomain/zero/1.0")) {
        license_url = "https://creativecommons.org/publicdomain/zero/1.0/";
      }
    } else if (cc_abbrev.size === 1) {
      const conditions = [...cc_abbrev.values()][0];
      license = `cc-${conditions}`;
      if (license in LICENSES) {
        license_url = `https://creativecommons.org/licenses/${conditions}/4.0/`;
      } else {
        console.warn(`Failed to detect CC license "${license}" (${log_ids})`);
        license = null;
      }
    }
    let org = { contact_url: "", full_name: "", local_name: "" };
    try {
      org = (await request(item.rightsHolder[0].link, "json")).orgs[0];
      if (org.contact_url && !org.contact_url.startsWith("http")) {
        org.contact_url = "https://" + org.contact_url;
      }
    } catch {
    }
    const attr_url = item.confidential === "true" ? org.contact_url : html_url;
    const date_archived = parseInt(item.dateArchived.slice(0, 4)) || 9999;
    const date_completed = parseInt(item.dateCompleted.slice(0, 4)) || 9999;
    const meta = {
      name: {
        local: item.nameCommonLocal,
        local_abbrev: item.nameAbbreviationLocal,
        english: item.nameCommon,
        english_abbrev: eng_abbrev.toUpperCase()
      },
      year: Math.min(date_archived, date_completed),
      direction: item.languageScriptDirection.toLowerCase() === "rtl" ? "rtl" : "ltr",
      copyright: {
        licenses: license ? [{ license, url: license_url }] : [],
        attribution: org.full_name || org.local_name || item.rightsHolder[0].name,
        attribution_url: attr_url
      },
      ids: {
        dbl: item.id
      },
      source: {
        service: "dbl",
        format: "usx",
        url: `https://app.thedigitalbiblelibrary.org/entry/download_archive?id=${item.id}&type=release`,
        updated: item.dateUpdated.slice(0, 10),
        revision: parseInt(item.revision)
      },
      literalness: null,
      tags: []
    };
    mkdirSync(trans_dir, { recursive: true });
    write_json(meta_file, meta, true);
    num_added += 1;
    existing.push(item.id);
  }));
  console.info(`DBL new: ${num_added}`);
  console.info(`DBL existing: ${num_existing}`);
  console.info(`DBL ignored: ${num_ignored}`);
  console.info(`DBL total: ${num_total}`);
}
export {
  discover
};
//# sourceMappingURL=dbl.js.map
