import { join } from "path";
import { existsSync, mkdirSync } from "fs";
import { LICENSES } from "../parts/license.js";
import { read_json, request, write_json } from "../parts/utils.js";
import { get_language_data } from "../parts/languages.js";
const IGNORE = [
  "fr_f10",
  // Sourced from eBible
  "my_juds",
  // Available on eBible
  "zh_cuv"
  // Sourced from eBible
];
function detect_license(rights, door43_id) {
  rights = rights.trim().toLowerCase();
  const src_license_url = `https://git.door43.org/Door43-Catalog/${door43_id}/src/branch/master/LICENSE.md`;
  if (rights.includes("public domain")) {
    return {
      license: "public",
      license_url: src_license_url
    };
  }
  const parts = rights.split(" ");
  const license = `${parts[0]}-${parts[1]}`;
  const version = parts[2];
  const license_url = `https://creativecommons.org/licenses/${parts[1]}/${version}/`;
  if (license in LICENSES) {
    return { license, license_url };
  }
  return { license: null, license_url: src_license_url };
}
function select_format(formats) {
  for (const format of formats) {
    if (format["format"].includes("application/zip") && format["format"].includes("text/usfm")) {
      return format;
    }
  }
  return null;
}
async function discover(existing, discover_specific_id) {
  const stats_bible = await _discover("Bible", existing, discover_specific_id);
  const stats_aligned = await _discover("Aligned_Bible", existing, discover_specific_id);
  const stats_hebrew = await _discover("Hebrew_Old_Testament", existing, discover_specific_id);
  console.info(`New: ${stats_bible.added + stats_aligned.added + stats_hebrew.added}`);
  console.info(`Existing: ${stats_bible.exists + stats_aligned.exists + stats_hebrew.exists}`);
}
async function _discover(subject, existing, discover_specific_id) {
  const catalog = await request(`https://api.door43.org/v3/subjects/${subject}.json`);
  const languages = get_language_data();
  const added = [];
  const exists = [];
  for (const language of catalog) {
    const lang_code = languages.normalise(language["language"]);
    if (!lang_code) {
      console.error(`IGNORED language ${language["language"]} (unknown code)`);
      continue;
    }
    for (const resource of language["resources"]) {
      const door43_id = `${language["language"]}_${resource["identifier"]}`;
      const trans_id = `${lang_code}_${resource["identifier"]}`;
      const log_ids = `${trans_id}/${door43_id}`;
      const trans_dir = join("sources", "bibles", trans_id);
      const meta_file = join(trans_dir, "meta.json");
      if (discover_specific_id && door43_id !== discover_specific_id) {
        continue;
      }
      if (existing.includes(door43_id)) {
        exists.push(door43_id);
        continue;
      }
      if (existsSync(meta_file)) {
        const existing_meta = read_json(meta_file);
        existing_meta.ids.door43 = door43_id;
        write_json(meta_file, existing_meta, true);
        exists.push(door43_id);
        continue;
      }
      if (IGNORE.includes(door43_id)) {
        console.warn(`IGNORED ${log_ids} (in ignore list)`);
        continue;
      } else if (resource["rights"].toLowerCase().includes("free translate")) {
        console.warn(`IGNORED ${log_ids} (restricted license)`);
        continue;
      }
      const { license, license_url } = detect_license(resource["rights"], door43_id);
      const format = select_format(resource["formats"]);
      if (!format) {
        console.error(`IGNORED ${log_ids} (unknown format)`);
        continue;
      }
      const year = parseInt(resource["issued"].slice(0, 4));
      const translation = {
        name: {
          local: "",
          local_abbrev: "",
          english: resource["title"],
          english_abbrev: resource["identifier"].toUpperCase()
        },
        year,
        direction: language["direction"] === "rtl" ? "rtl" : "ltr",
        copyright: {
          attribution: `\xA9 ${year} ${resource["creator"]}`,
          // Using git URL since site doesn't have full collection (including non-door43)
          attribution_url: `https://git.door43.org/Door43-Catalog/${door43_id}`,
          licenses: license ? [{ license, url: license_url }] : []
        },
        ids: {
          door43: door43_id
        },
        source: {
          service: "door43",
          format: "usfm",
          url: format["url"],
          updated: format["modified"].split("T")[0],
          revision: 0
          // Unused
        },
        literalness: null,
        tags: []
      };
      mkdirSync(trans_dir, { recursive: true });
      write_json(meta_file, translation, true);
      added.push(door43_id);
    }
  }
  return { added: added.length, exists: exists.length };
}
export {
  discover
};
//# sourceMappingURL=door43.js.map
