import { join } from "path";
import { existsSync, mkdirSync } from "fs";
import { parse as csv_parse } from "csv-parse/sync";
import { request, concurrent, write_json, read_json } from "../parts/utils.js";
import { LICENSES, detect_year } from "../parts/license.js";
import { get_language_data } from "../parts/languages.js";
const IGNORE = [
  // Ignored for being a duplicate
  "eng-kjv",
  // Same as 'eng-kjv2006' with Apoc.
  "eng-web",
  // Copy of existing translation with added deuterocanon
  "eng-webbe",
  // Copy of existing translation with added deuterocanon
  "eng-web-c",
  // Copy of existing translation with added deuterocanon
  "aka",
  // An incorrect mix of twiasante and twi, one of which has same text
  "hbo",
  // Assumed to be same text as hboWLC
  "pon2006a",
  // Same as pon2006 but with Apoc.
  "hltmcsb",
  // 'hlt' is the same but includes strongs
  // Ignored for being too questionable
  "spav1602p",
  // A KJV update of the original Valera from 1602
  // Ignored for not having a complete book (and unlikely to ever do so)
  "kld",
  // Old collection of snippets, not complete books
  "alh",
  // Songs, not biblical books
  "bbd",
  // Just a Bible story, not Bible book
  "yij",
  // Partial book only
  "rmb",
  // Partial book only
  "lrg",
  // One passage only
  "kux",
  // Two passages only
  "kky",
  // One passage only
  "kjn",
  // One passage only
  "gupmay",
  // One passage only
  "dwuliv",
  // One passage only
  "adt",
  // Two passages only
  "byr-w",
  // Partial books only
  "djj",
  // Partial books only (2023, so may get more)
  "guf",
  // Partial books only (2023, so may get more)
  "gupk",
  // Partial books only
  "nay",
  // Partial books only
  "omw-v",
  // Partial books only
  "tbg-a",
  // Partial books only
  "tiwm",
  // Single passage only (2023, so may get more)
  "wlg"
  // Partial books only
];
const IGNORE_UNLICENSED = [
  "daf",
  "dud",
  "kud2014",
  "bbr2013",
  "big2013",
  "fad",
  "ino2013",
  "kxw",
  "nuq"
];
async function discover(existing, discover_specific_id) {
  const csv_buffer = await request("https://ebible.org/Scriptures/translations.csv", "arrayBuffer");
  const rows = csv_parse(Buffer.from(csv_buffer), {
    bom: true,
    columns: true,
    skip_empty_lines: true,
    trim: true
  });
  const language_data = get_language_data();
  const added = [];
  const exists = [];
  const ignored = [];
  await concurrent(rows.map((row) => async () => {
    const ebible_id = row["translationId"];
    const ebible_url = `https://ebible.org/Scriptures/details.php?id=${ebible_id}`;
    const lang_code = language_data.normalise(row["languageCode"]);
    let fcbhid_end = row["FCBHID"].slice(3).toLowerCase();
    if (fcbhid_end === "twf") {
      fcbhid_end = "wft";
    }
    let trans_id = `${lang_code ?? ""}_${fcbhid_end}`;
    const log_ids = `${trans_id}/${ebible_id}`;
    if (discover_specific_id && ebible_id !== discover_specific_id) {
      return;
    }
    if (IGNORE.includes(ebible_id)) {
      ignored.push(ebible_id);
      return;
    }
    if (existing.includes(ebible_id)) {
      exists.push(ebible_id);
      return;
    }
    if (!lang_code || !fcbhid_end) {
      trans_id = "_" + row["FCBHID"].toLowerCase();
      console.error(`INVALID Id ${trans_id} ${ebible_url}`);
    }
    let page_resp = await request(ebible_url, "text");
    page_resp = page_resp.replaceAll("/by4.0/", "/by/4.0/");
    let license = null;
    let license_url = ebible_url;
    const cc_url = /creativecommons.org\/licenses\/([a-z-]+)\/(\d+\.\d+)/i.exec(page_resp);
    if (cc_url) {
      license = `cc-${cc_url[1]}`;
      if (license in LICENSES) {
        license_url = `https://creativecommons.org/licenses/${cc_url[1]}/${cc_url[2]}/`;
      } else {
        console.warn(`Failed to detect CC license "${license}" (${log_ids})`);
        license = null;
      }
    } else if (/public domain/i.test(page_resp) && !/not public domain/i.test(page_resp)) {
      license = "public";
    } else if (IGNORE_UNLICENSED.includes(ebible_id)) {
      return;
    }
    if (!page_resp.includes("usfm.zip")) {
      if (license) {
        console.error(`INVALID No USFM available ${ebible_url}`);
      }
      ignored.push(ebible_id);
      return;
    }
    const trans_dir = join("sources", "bibles", trans_id);
    const meta_file = join(trans_dir, "meta.json");
    if (existsSync(meta_file)) {
      const existing_meta = read_json(meta_file);
      if (existing_meta.ids.ebible) {
        console.error(`INVALID Matched ${ebible_id} to ${trans_id} but it already has ` + existing_meta.ids.ebible);
        return;
      }
      existing_meta.ids.ebible = ebible_id;
      write_json(meta_file, existing_meta, true);
      exists.push(ebible_id);
      return;
    }
    const english_name = row["shortTitle"].normalize("NFKD").replace(/\p{Diacritic}/gu, "");
    const english_abbrev = english_name.replace(/[^A-Z]/g, "").slice(0, 5);
    const meta = {
      name: {
        local: row["title"],
        // Usually in local language
        local_abbrev: "",
        english: lang_code === "eng" ? row["title"] : english_name,
        english_abbrev
      },
      year: detect_year(
        row["title"],
        row["shortTitle"],
        row["translationId"],
        row["swordName"],
        row["Copyright"],
        row["description"]
      ),
      direction: row["textDirection"] === "rtl" ? "rtl" : "ltr",
      copyright: {
        licenses: license ? [{ license, url: license_url }] : [],
        attribution: row["Copyright"],
        // Use eBible URL so can point to specific translation and not generic org
        attribution_url: ebible_url
      },
      ids: {
        ebible: ebible_id
      },
      source: {
        service: "ebible",
        format: "usfm",
        url: `https://ebible.org/Scriptures/${ebible_id}_usfm.zip`,
        updated: row["UpdateDate"],
        revision: 0
        // Unused
      },
      literalness: null,
      tags: []
    };
    mkdirSync(trans_dir, { recursive: true });
    write_json(meta_file, meta, true);
    added.push(ebible_id);
  }));
  console.info(`EBIBLE new: ${added.length}`);
  console.info(`EBIBLE existing: ${exists.length}`);
  console.info(`EBIBLE ignored: ${ignored.length}`);
  console.info(`EBIBLE total: ${added.length + exists.length + ignored.length}`);
}
export {
  discover
};
//# sourceMappingURL=ebible.js.map
