import { join } from "node:path";
import { writeFileSync } from "node:fs";
import StreamZip from "node-stream-zip";
import { mkdir_exist, request } from "../parts/utils.js";
import { books_ordered } from "../parts/bible.js";
async function download_data() {
  mkdir_exist(join("sources", "crossref"));
  const zip_path = join("sources", "crossref", "cross_references.zip");
  const zip = await request("https://a.openbible.info/data/cross-references.zip", "arrayBuffer");
  writeFileSync(zip_path, Buffer.from(zip));
  const extractor = new StreamZip.async({ file: zip_path });
  const data = await extractor.entryData("cross_references.txt");
  return cross_references_to_json(data.toString("utf8"));
}
function cross_references_to_json(content) {
  const output = {};
  for (const usx_book of Object.values(openbible_to_usx_book)) {
    output[usx_book] = {};
  }
  const cross_ref_table = /* @__PURE__ */ new Map();
  const total_votes = [];
  const lines = content.split("\n");
  for (let index = 0; index < lines.length; index++) {
    const line = lines[index];
    if (index === 0 || !line) {
      continue;
    }
    const parts = line.split("	");
    const from = extract_reference(parts[0]);
    const to = extract_reference(parts[1]);
    if (!from || !to) {
      continue;
    }
    const votes_string = parts[2] || "0";
    const votes = parseInt(votes_string, 10);
    if (votes < 0) {
      continue;
    }
    total_votes.push(votes);
    let cross_ref = [
      votes,
      to.usx,
      to.start_chapter,
      to.start_verse
    ];
    if (to.is_range) {
      cross_ref = [...cross_ref, to.end_chapter, to.end_verse];
    }
    const key = `${from.usx}_${from.start_chapter}_${from.start_verse}`;
    if (cross_ref_table.has(key)) {
      const existing = cross_ref_table.get(key);
      existing.cross_references.push(cross_ref);
      cross_ref_table.set(key, existing);
      continue;
    }
    const item = {
      from,
      cross_references: [cross_ref]
    };
    cross_ref_table.set(key, item);
  }
  total_votes.sort((a, b) => a - b);
  const group_size = Math.ceil(total_votes.length / 3);
  const vote_groups = Array.from({ length: 3 }, (_, index) => {
    return total_votes.slice(index * group_size, (index + 1) * group_size);
  });
  cross_ref_table.forEach((item) => {
    item.cross_references = item.cross_references.map((ref) => {
      const vote = ref[0];
      if (vote_groups[0].includes(vote)) {
        ref[0] = 3;
      } else if (vote_groups[1].includes(vote)) {
        ref[0] = 2;
      } else {
        ref[0] = 1;
      }
      return ref;
    }).sort(
      (a, b) => {
        const a_index = books_ordered.indexOf(a[1]);
        const b_index = books_ordered.indexOf(b[1]);
        const a_chapter = a[2];
        const b_chapter = b[2];
        const a_verse = a[3];
        const b_verse = b[3];
        if (a_index === b_index) {
          if (a_chapter === b_chapter) {
            return a_verse - b_verse;
          }
          return a_chapter - b_chapter;
        }
        return a_index - b_index;
      }
    );
    const from = item.from;
    if (!(from.start_chapter in output[from.usx])) {
      output[from.usx][from.start_chapter] = {};
    }
    if (!(from.start_verse in output[from.usx][from.start_chapter])) {
      output[from.usx][from.start_chapter][from.start_verse] = [];
    }
    output[from.usx][from.start_chapter][from.start_verse] = item.cross_references;
  });
  return output;
}
function extract_reference(reference) {
  const pattern = /\b\w+\b/g;
  const parts = reference.split("-");
  if (parts.length === 0) {
    return null;
  }
  const matches = parts[0].match(pattern) || [];
  if (matches.length < 3) {
    return null;
  }
  const book = matches[0] || "";
  const usx = openbible_to_usx_book[book] || "";
  const start_chapter = parseInt(matches[1], 10) || 0;
  const start_verse = parseInt(matches[2], 10) || 0;
  let end_chapter = start_chapter;
  let end_verse = start_verse;
  if (parts.length === 2) {
    const second_matches = parts[1].match(pattern) || [];
    if (second_matches.length === 3) {
      end_chapter = parseInt(second_matches[1], 10) || 0;
      end_verse = parseInt(second_matches[2], 10) || 0;
    }
  }
  const is_range = start_chapter !== end_chapter || start_verse !== end_verse;
  const result = {
    book,
    usx,
    start_chapter,
    start_verse,
    end_chapter,
    end_verse,
    is_range
  };
  return result;
}
const openbible_to_usx_book = {
  "Gen": "gen",
  "Exod": "exo",
  "Lev": "lev",
  "Num": "num",
  "Deut": "deu",
  "Josh": "jos",
  "Judg": "jdg",
  "Ruth": "rut",
  "1Sam": "1sa",
  "2Sam": "2sa",
  "1Kgs": "1ki",
  "2Kgs": "2ki",
  "1Chr": "1ch",
  "2Chr": "2ch",
  "Ezra": "ezr",
  "Neh": "neh",
  "Esth": "est",
  "Job": "job",
  "Ps": "psa",
  "Prov": "pro",
  "Eccl": "ecc",
  "Song": "sng",
  "Isa": "isa",
  "Jer": "jer",
  "Lam": "lam",
  "Ezek": "ezk",
  "Dan": "dan",
  "Hos": "hos",
  "Joel": "jol",
  "Amos": "amo",
  "Obad": "oba",
  "Jonah": "jon",
  "Mic": "mic",
  "Nah": "nam",
  "Hab": "hab",
  "Zeph": "zep",
  "Hag": "hag",
  "Zech": "zec",
  "Mal": "mal",
  "Matt": "mat",
  "Mark": "mrk",
  "Luke": "luk",
  "John": "jhn",
  "Acts": "act",
  "Rom": "rom",
  "1Cor": "1co",
  "2Cor": "2co",
  "Gal": "gal",
  "Eph": "eph",
  "Phil": "php",
  "Col": "col",
  "1Thess": "1th",
  "2Thess": "2th",
  "1Tim": "1ti",
  "2Tim": "2ti",
  "Titus": "tit",
  "Phlm": "phm",
  "Heb": "heb",
  "Jas": "jas",
  "1Pet": "1pe",
  "2Pet": "2pe",
  "1John": "1jn",
  "2John": "2jn",
  "3John": "3jn",
  "Jude": "jud",
  "Rev": "rev"
};
export {
  cross_references_to_json,
  download_data,
  extract_reference,
  openbible_to_usx_book
};
//# sourceMappingURL=openbible.js.map
