import { join } from "node:path";
import { existsSync, readFileSync } from "node:fs";
import { DOMParser, XMLSerializer } from "@xmldom/xmldom";
import { select } from "xpath";
function get_notes(id) {
  const notes_path = join("sources", "notes", id, "StudyNotes.xml");
  if (!existsSync(notes_path)) {
    return null;
  }
  return study_notes_to_json(readFileSync(notes_path, { encoding: "utf8" }));
}
function study_notes_to_json(xml) {
  const output = {};
  for (const usx_book of Object.values(tyndale_to_usx_book)) {
    output[usx_book] = {
      verses: {},
      ranges: []
    };
  }
  const doc = new DOMParser().parseFromString(xml, "text/xml");
  const elements = select("//item", doc);
  const serializer = new XMLSerializer();
  elements.forEach((element) => {
    const name = element.getAttribute("name") ?? "";
    const reference = extract_reference(name);
    if (!reference || reference.usx === "") {
      return;
    }
    const body_element = select("body/p", element)[0];
    let body = body_element !== void 0 ? serializer.serializeToString(body_element) : "";
    body = clean_note(body);
    if (!reference.is_range) {
      if (!(reference.start_chapter in output[reference.usx].verses)) {
        output[reference.usx].verses[reference.start_chapter] = {};
      }
      output[reference.usx].verses[reference.start_chapter][reference.start_verse] = body;
    } else {
      const note = {
        start_c: reference.start_chapter,
        start_v: reference.start_verse,
        end_c: reference.end_chapter,
        end_v: reference.end_verse,
        contents: body
      };
      output[reference.usx].ranges.push(note);
    }
  });
  for (const book in output) {
    if (!output[book].ranges.length && !Object.keys(output[book].verses).length) {
      delete output[book];
    }
  }
  for (const key in output) {
    output[key].ranges.sort((a, b) => {
      return a.start_c - b.start_c || a.start_v - b.start_v;
    });
  }
  return output;
}
function extract_reference(reference) {
  const regex = /\b\w+\b/g;
  const parts = reference.split("-");
  if (parts.length === 0) {
    return null;
  }
  const matches = parts[0].match(regex) || [];
  if (matches.length < 3) {
    return null;
  }
  let book = matches[0] || "";
  let usx = tyndale_to_usx_book[book] || "";
  if (usx === "") {
    book = refs_to_tyndale[book] || "";
    usx = tyndale_to_usx_book[book] || "";
  }
  const start_chapter = parseInt(matches[1], 10) || 0;
  const start_verse = parseInt(matches[2], 10) || 0;
  let end_chapter = 0;
  let end_verse = 0;
  if (parts.length === 2) {
    const matches2 = parts[1].match(regex) || [];
    if (matches2.length === 2) {
      end_chapter = parseInt(matches2[0], 10) || 0;
      end_verse = parseInt(matches2[1], 10) || 0;
    } else {
      end_chapter = start_chapter;
      end_verse = parseInt(matches2[0], 10) || 0;
    }
  } else {
    end_chapter = start_chapter;
    end_verse = start_verse;
  }
  const is_range = start_chapter !== end_chapter || start_verse !== end_verse;
  const result = {
    book,
    start_chapter,
    start_verse,
    end_chapter,
    end_verse,
    usx,
    is_range
  };
  return result;
}
function clean_note(body) {
  let cleaned = body;
  cleaned = cleaned.replace(/^<p(?:\s+class="[^"]+")?>([\s\S]+)<\/p>$/, "$1");
  cleaned = cleaned.replace(/<span class="sn-ref">.*?<\/span>/g, "");
  cleaned = cleaned.replace(/<span class="(.*?)">/g, '<span class="fbn-$1">');
  const elements = {
    "bold": "<strong>$1</strong>",
    "bold-era": '<strong class="fbn-era">$1</strong>',
    "bold-sc": '<strong class="fbn-sc">$1</strong>',
    "sn-excerpt": '<span class="fbn-excerpt">$1</span>',
    "sn-excerpt-divine-name": '<span class="fbn-excerpt fbn-yhwh">$1</span>',
    "sn-excerpt-roman": '<span class="fbn-excerpt fbn-roman">$1</span>',
    "sn-excerpt-sc": '<span class="fbn-excerpt fbn-sc">$1</span>',
    "sn-hebrew-chars": '<span class="fbn-hebrew">$1</span>',
    "sn-list-1": '<span class="fbn-list-1">$1</span>',
    "sn-list-2": '<span class="fbn-list-2">$1</span>',
    "sn-list-3": '<span class="fbn-list-3">$1</span>',
    "sn-ref-sc": '<span class="fbn-sc">$1</span>',
    "sn-sc": '<span class="fbn-sc">$1</span>',
    "divine-name-ital": '<em class="fbn-yhwh">$1</em>',
    "ital": "<em>$1</em>",
    "ital-bold": "<em><strong>$1</strong></em>",
    "sc-ital": '<em class="fbn-sc">$1</em>',
    "sub": "<sub>$1</sub>",
    "sup": "<sup>$1</sup>"
  };
  Object.entries(elements).forEach(([klass, replacement]) => {
    const pattern2 = new RegExp(`<span class="fbn-${klass}">(.*?)</span>`, "g");
    cleaned = cleaned.replace(pattern2, replacement);
  });
  let pattern = /<a href="\?bref=([^"]*)">([^<]*)<\/a>/g;
  cleaned = cleaned.replace(pattern, (match, reference, text) => {
    const scripture = extract_reference(reference);
    if (!scripture) {
      return "";
    }
    let format = `${scripture.usx},${scripture.start_chapter},${scripture.start_verse}`;
    if (scripture.is_range) {
      format += `,${scripture.end_chapter},${scripture.end_verse}`;
    }
    return `<span data-ref="${format}">${text}</span>`;
  });
  pattern = /<a\b[^>]*>(.*?)<\/a>/gi;
  cleaned = cleaned.replace(pattern, "$1");
  return cleaned.trim();
}
const refs_to_tyndale = {
  "1Sam": "ISam",
  "2Sam": "IISam",
  "1Kgs": "IKgs",
  "2Kgs": "IIKgs",
  "1Chr": "IChr",
  "2Chr": "IIChr",
  "1Cor": "ICor",
  "2Cor": "IICor",
  "1Thes": "IThes",
  "2Thes": "IIThes",
  "1Tim": "ITim",
  "2Tim": "IITim",
  "1Pet": "IPet",
  "2Pet": "IIPet",
  "1Jn": "IJn",
  "2Jn": "IIJn",
  "3Jn": "IIIJn"
};
const tyndale_to_usx_book = {
  "Gen": "gen",
  "Exod": "exo",
  "Lev": "lev",
  "Num": "num",
  "Deut": "deu",
  "Josh": "jos",
  "Judg": "jdg",
  "Ruth": "rut",
  "ISam": "1sa",
  "IISam": "2sa",
  "IKgs": "1ki",
  // WARN Tyndale uses 'IKgs' in `name`, not '1Kgs' as in `<refs>` (use 'IKgs')
  "IIKgs": "2ki",
  "IChr": "1ch",
  "IIChr": "2ch",
  "Ezra": "ezr",
  "Neh": "neh",
  "Esth": "est",
  "Job": "job",
  "Ps": "psa",
  "Pr": "pro",
  "Eccl": "ecc",
  "Song": "sng",
  "Isa": "isa",
  "Jer": "jer",
  "Lam": "lam",
  "Ezek": "ezk",
  "Dan": "dan",
  "Hos": "hos",
  "Joel": "jol",
  "Amos": "amo",
  "Obad": "oba",
  "Jon": "jon",
  "Mic": "mic",
  "Nah": "nam",
  "Hab": "hab",
  "Zeph": "zep",
  "Hagg": "hag",
  "Zech": "zec",
  "Mal": "mal",
  "Matt": "mat",
  "Mark": "mrk",
  "Luke": "luk",
  "John": "jhn",
  "Acts": "act",
  "Rom": "rom",
  "ICor": "1co",
  "IICor": "2co",
  "Gal": "gal",
  "Eph": "eph",
  "Phil": "php",
  "Col": "col",
  "IThes": "1th",
  "IIThes": "2th",
  "ITim": "1ti",
  "IITim": "2ti",
  "Titus": "tit",
  "Phlm": "phm",
  "Heb": "heb",
  "Jas": "jas",
  "IPet": "1pe",
  "IIPet": "2pe",
  "IJn": "1jn",
  "IIJn": "2jn",
  "IIIJn": "3jn",
  "Jude": "jud",
  "Rev": "rev"
};
export {
  clean_note,
  extract_reference,
  get_notes,
  refs_to_tyndale,
  study_notes_to_json,
  tyndale_to_usx_book
};
//# sourceMappingURL=tyndale.js.map
