import { join } from "path";
import { existsSync } from "fs";
import { merge } from "lodash-es";
import { PKG_PATH, read_json, request, write_json } from "./utils.js";
const overrides = {
  languages: {
    /* Chinese is hard to classify
        Many in China speak Mandarin but read simplified script
        Many in Taiwan speak Mandarin but read traditional script
        Many in Hong Kong speak Cantonese but read traditional script
        Many in Singapore speak Hokkien but read simplified script
    International codes and this platform learn towards classifying languages by speech
    Audio bibles can be assigned the correct spoken language
    But written Chinese bibles can't be isolated to a single spoken language
    TODO Solution: client-side aliasing (for now just putting everything in "cmn")
    */
    cmn: { local: "\u4E2D\u6587", english: "Chinese" },
    // TODO Change to "Mandarin" when aliasing done
    awk: { local: "Awabakal" }
    // Population data uses region name by mistake, no CLDR data
  },
  language2to3: {
    // TODO Modify to support including country codes and routing to correct spoken language
    zh: "cmn"
    // Default to cmn for now which contains all Chinese bibles
  }
};
async function gen_language_data() {
  let cldr_path = join(PKG_PATH, "node_modules", "cldr-localenames-full", "main");
  if (!existsSync(cldr_path)) {
    cldr_path = join(PKG_PATH, "..", "node_modules", "cldr-localenames-full", "main");
  }
  const data = { languages: {}, language2to3: {} };
  const en_path = join(cldr_path, "en", "languages.json");
  const english = read_json(en_path).main["en"].localeDisplayNames.languages;
  const population = read_json(join(PKG_PATH, "dist", "data", "population.json"));
  const iso_url = "https://iso639-3.sil.org/sites/iso639-3/files/downloads/iso-639-3.tab";
  const codes = await request(iso_url, "text");
  for (const language of codes.trim().split("\n").slice(1)) {
    const parts = language.trim().split("	");
    const three = parts[0];
    const two = parts[3];
    const pop = parts[5] !== "L" ? null : population[three]?.pop ?? 0;
    const label = parts[6];
    const english_name = english[three] || english[two] || population[three]?.english || label;
    data.languages[three] = {
      local: population[three]?.local || english_name,
      english: english_name,
      pop
    };
    if (two.length === 2) {
      data.language2to3[two] = three;
    }
    let cldr_code = three;
    let lang_path = join(cldr_path, cldr_code, "languages.json");
    if (!existsSync(lang_path) && two) {
      cldr_code = two;
      lang_path = join(cldr_path, cldr_code, "languages.json");
    }
    if (existsSync(lang_path)) {
      data.languages[three].local = read_json(lang_path).main[cldr_code].localeDisplayNames.languages[cldr_code];
    }
  }
  merge(data, overrides);
  write_json(join("sources", "languages.json"), data, true);
}
class Languages {
  // An interface for interacting with language data
  data;
  constructor(data) {
    this.data = data;
  }
  normalise(code) {
    code = code.split("-")[0].toLocaleLowerCase();
    return this.data.language2to3[code] ?? (code in this.data.languages ? code : null);
  }
}
function get_language_data() {
  return new Languages(read_json(join("sources", "languages.json")));
}
export {
  Languages,
  gen_language_data,
  get_language_data
};
//# sourceMappingURL=languages.js.map
