import { join } from "path";
import { existsSync } from "fs";
import { book_names_english, books_ordered } from "./bible.js";
import { languages_by_total_speakers } from "../data/languages.js";
import { get_language_data } from "./languages.js";
import { LICENSES } from "./license.js";
import { read_json, write_json, list_dirs, list_files } from "./utils.js";
function _missing_meta(meta) {
  return !meta.year || !meta.copyright.licenses.length || !(meta.name.local || meta.name.english) || !(meta.name.local_abbrev || meta.name.english_abbrev);
}
async function update_manifest() {
  console.info("Generating manifest...");
  const manifest = {
    translations: {},
    languages: {},
    language2to3: {},
    languages_most_spoken: [],
    books_ordered,
    book_names_english,
    licenses: LICENSES
  };
  const included_languages = /* @__PURE__ */ new Set();
  for (const trans of list_dirs(join("dist", "bibles"))) {
    if (trans === "manifest.json") {
      continue;
    }
    const meta = read_json(join("sources", "bibles", trans, "meta.json"));
    if (_missing_meta(meta)) {
      console.error(`IGNORING ${trans} (missing year, license, name, etc)`);
      continue;
    }
    const html_dir = join("dist", "bibles", trans, "html");
    const html_books = existsSync(html_dir) ? list_files(html_dir).map((name) => name.slice(0, 3)) : [];
    if (html_books.length === 0) {
      console.error(`IGNORING ${trans} (no books)`);
      continue;
    }
    const books_ot = books_ordered.slice(0, 39).filter((b) => html_books.includes(b));
    const books_nt = books_ordered.slice(39).filter((b) => html_books.includes(b));
    manifest.translations[trans] = {
      name: meta.name,
      year: meta.year,
      // Verified to exist above
      direction: meta.direction,
      copyright: meta.copyright,
      literalness: meta.literalness,
      tags: meta.tags,
      // Record as `true` if whole testament to reduce data size
      books_ot: books_ot.length === 39 ? true : books_ot,
      books_nt: books_nt.length === 27 ? true : books_nt
    };
    included_languages.add(trans.slice(0, 3));
  }
  const language_data = get_language_data();
  manifest.languages = Object.fromEntries(Object.entries(language_data.data.languages).filter(([code]) => included_languages.has(code)));
  manifest.language2to3 = Object.fromEntries(Object.entries(language_data.data.language2to3).filter(([, three]) => included_languages.has(three)));
  manifest.languages_most_spoken = languages_by_total_speakers.filter((l) => included_languages.has(l));
  write_json(join("dist", "bibles", "manifest.json"), manifest);
}
export {
  _missing_meta,
  update_manifest
};
//# sourceMappingURL=manifest.js.map
