import {
  existsSync,
  mkdirSync,
  readFileSync,
  readdirSync,
  rmSync,
  statSync,
  writeFileSync
} from "fs";
import { dirname, join } from "path";
const IGNORE_FILES = [".DS_Store"];
async function request(url, type = "json") {
  const resp = await fetch(url);
  if (!resp.ok) {
    throw new Error(`${resp.status} ${resp.statusText}: ${url}`);
  }
  return resp[type]();
}
async function concurrent(tasks, limit = 10) {
  const channels = [...Array(limit).keys()].map((i) => Promise.resolve(i));
  for (const task of tasks) {
    const free_channel = await Promise.race(channels);
    channels[free_channel] = task().then(() => {
      return free_channel;
    });
  }
  await Promise.all(channels);
}
function mkdir_exist(path) {
  if (existsSync(path)) {
    return;
  }
  mkdirSync(path, { recursive: true });
}
function clean_dir(path) {
  rmSync(path, { force: true, recursive: true });
  mkdirSync(path, { recursive: true });
}
function read_dir(path) {
  return readdirSync(path).filter((item) => !IGNORE_FILES.includes(item));
}
function get_dir_entries(path) {
  return readdirSync(path, { withFileTypes: true }).filter((item) => !IGNORE_FILES.includes(item.name)).map((item) => {
    const name = item.name;
    const isDirectory = item.isDirectory();
    const fileSize = isDirectory ? void 0 : statSync(join(path, name)).size;
    const dirSize = !isDirectory ? void 0 : readdirSync(join(path, name)).length;
    const entry = {
      isDirectory,
      name,
      fileSize,
      dirSize
    };
    return entry;
  });
}
function list_files(directory) {
  return readdirSync(directory, { withFileTypes: true }).filter((entity) => entity.isFile()).filter((entity) => !IGNORE_FILES.includes(entity.name)).map((entity) => entity.name);
}
function list_dirs(directory) {
  return readdirSync(directory, { withFileTypes: true }).filter((entity) => entity.isDirectory()).map((entity) => entity.name);
}
function read_dir_deep(directory) {
  const dirs = [directory];
  const files = [];
  for (const entry of readdirSync(directory, { withFileTypes: true })) {
    const entry_path = join(directory, entry.name);
    if (entry.isDirectory()) {
      const subdir_results = read_dir_deep(entry_path);
      dirs.push(...subdir_results.dirs);
      files.push(...subdir_results.files);
      continue;
    } else if (!entry.isFile() || IGNORE_FILES.includes(entry.name)) {
      continue;
    }
    files.push(entry_path);
  }
  return { dirs, files };
}
function read_json(path) {
  return JSON.parse(readFileSync(path, "utf-8"));
}
function write_json(path, data, indented = false) {
  writeFileSync(path, JSON.stringify(data, void 0, indented ? 4 : 0));
}
const PKG_PATH = dirname(dirname(dirname(join("/", ...import.meta.url.slice("file:/".length).split("/")))));
function type_from_path(path) {
  if (path.endsWith(".usx")) {
    return "application/xml";
  } else if (path.endsWith(".usfm")) {
    return "text/plain";
  } else if (path.endsWith(".html")) {
    return "text/html";
  } else if (path.endsWith(".txt")) {
    return "text/plain";
  } else if (path.endsWith(".json")) {
    return "application/json";
  }
  return "application/octet-stream";
}
export {
  PKG_PATH,
  clean_dir,
  concurrent,
  get_dir_entries,
  list_dirs,
  list_files,
  mkdir_exist,
  read_dir,
  read_dir_deep,
  read_json,
  request,
  type_from_path,
  write_json
};
//# sourceMappingURL=utils.js.map
